﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Data;
using System;
using System.Linq;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Problems.TestFunctions.MultiObjective;

namespace HeuristicLab.Algorithms.MOCMAEvolutionStrategy {
  [Item("CrowdingIndicator", "Selection of Offspring based on CrowdingDistance")]
  [StorableClass]
  class CrowdingIndicator : Item, IIndicator {


    public int LeastContributer<R>(R[] front, Func<R, double[]> extractor, MultiObjectiveTestFunctionProblem problem) {
      DoubleMatrix bounds = problem.Bounds;
      int mindex = 1;
      double min = Double.MaxValue;
      double[] pointsums = new double[front.Count()];

      for (int dim = 0; dim < problem.Objectives; dim++) {
        double[] arr = front.Select(x => extractor.Invoke(x)[dim]).ToArray();
        Array.Sort(arr);
        double fmax = problem.Bounds[dim % bounds.Rows, 1];
        double fmin = bounds[dim % bounds.Rows, 0];
        int pointIdx = 0;
        foreach (R point in front) {
          double d = 0;
          int pos = Array.BinarySearch(arr, extractor.Invoke(point)[dim]);

          d = pos != 0 && pos != arr.Count() - 1 ? (arr[pos + 1] - arr[pos - 1]) / (fmax - fmin) : Double.PositiveInfinity;
          pointsums[pointIdx] += d;


          pointIdx++;
        }
      }
      //find min
      for (int pointIdx = 0; pointIdx < pointsums.Length; pointIdx++) {
        double d = pointsums[pointIdx];
        if (!Double.IsInfinity(d) && min > d) {
          mindex = pointIdx;
          min = d;
        }
      }

      return mindex;
    }



    public override IDeepCloneable Clone(Cloner cloner) {
      return new CrowdingIndicator(this, cloner);
    }

    [StorableConstructor]
    protected CrowdingIndicator(bool deserializing) : base(deserializing) { }
    protected CrowdingIndicator(CrowdingIndicator original, Cloner cloner)
      : base(original, cloner) {
    }
    public CrowdingIndicator() { }
  }
}
